/****************************************************************************
**
** Copyright (C) 2023 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of the Qt Design Viewer of the Qt Toolkit.
**
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3 as published by the Free Software
** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-3.0.html.
**
****************************************************************************/

#ifndef PROJECTMANAGER_H
#define PROJECTMANAGER_H

#include <QJsonObject>
#include <QObject>
#include <QQmlComponent>
#include <QQmlEngine>
#include <QQuickView>
#include <QQuickWindow>

class ProjectManager : public QObject
{
    Q_OBJECT
public:
    explicit ProjectManager(QObject *parent = nullptr);
    ~ProjectManager();

    QString unpackProject(const QByteArray &project, bool extractZip = false);
    bool runProject(const QString &projectPath);

    bool cacheProject(const QByteArray &projectData, const QJsonObject &projectInfo);
    bool isProjectCached(const QJsonObject &projectInfo);
    bool runCachedProject(const QJsonObject &projectInfo);

    bool cacheDemoProject(const QByteArray &projectData, const QJsonObject &projectInfo);
    bool isDemoProjectCached(const QJsonObject &projectName);
    bool runDemoProject(const QString &projectName);
    void clearDemoCaches();

    void showAppWindow();
    void hideAppWindow();

public slots:
    void orientateWindow(Qt::ScreenOrientation orientation);

private:
    // Member variables
    QByteArray m_projectData;
    QString m_projectPath;
    const QString m_projectCachePath;
    const QString m_demoProjectCachePath;

    // Qml related members
    QScopedPointer<QQmlEngine> m_qmlEngine;
    QScopedPointer<QQmlComponent> m_qmlComponent;
    QScopedPointer<QQuickWindow> m_quickWindow;

    // Member functions
    QString findFile(const QString &dir, const QString &filter);
    QString readQmlProjectFile(const QString &qmlProjectFilePath);
    QString getMainQmlFile(const QString &projectPath, const QString &qmlProjectFileContent);
    QStringList getImportPaths(const QString &projectPath, const QString &qmlProjectFileContent);
    bool isQt6Project(const QString &qmlProjectFileContent);

    void parseQmlProjectFile(const QString &fileName, QString *mainFile, QStringList *importPaths);
    void cacheProject(const QString &projectName, const QString &projectPath);
    void cleanupResources();

signals:
    void closingProject();
};

#endif // PROJECTMANAGER_H
